import rp from 'request-promise'
import dotenv from 'dotenv'

dotenv.config({silent:true})

class Base {
    constructor(
        id = 0, 
        parent = {}, 
        api_url = 'http://127.0.0.1:8000', 
        fetch = false
    ) {
        this.api_url = process.env.API_URL || api_url
        this.id = id
        this.name = this.get_name()
        this.parent = parent
        this.storage = []
        this.url = ''
        this.fetch = fetch
        this.object = {}
    }

    clear() {
        this.object = {}
        this.id = 0
        this.url = ''
    }

    fetch_resource(id) {
        return this.get(id).then((data) => {
            let record = data[0]
            Object.assign(this.object, record)
        })
    }

    get_name() {
        return this.process_class_name(this.constructor.name)
    }

    get_resource_path(id = 0) {
        id = id || this.id
        var url_str = `${this.name}`
        if(id) {
            url_str += `/${id}`
        }
        return url_str
    }

    process_class_name(name) {
        return `${name}s`.toLowerCase()
    }

    construct_url(id = 0, parent = {}) {
        var url_str = ''
        if(this.storage.length === 0) {
            this.storage.push(this.get_resource_path(id))
        }
        if(typeof(parent.id) !== 'undefined') {
            this.storage.push(parent.get_resource_path())
            return this.construct_url(0, parent.parent)
        }
        url_str = this.storage.reverse().join('/')
        this.storage = []
        return url_str
    }

    perform_request(method, id=0, data={}) {
        var url = `${this.api_url}/${this.construct_url(id, this.parent)}/`

        return rp({
            method: method,
            uri: url,
            body: data,
            json: true
        })
    }

    get(id = 0) {
        return this.perform_request(
            'GET', 
            id
        )
    }

    post(data = {}) {
        return this.perform_request(
            'POST',
            0,
            data
        )
    }

    put(id = 0, data = {}) {

        if(this.fetch && !$.isEmptyObject(this.object)) {
            data = this.object
        }

        return this.perform_request(
            'PUT', 
            id,
            data
        )
    }

    patch(id = 0, data = {}) {

        if(this.fetch && data === {}) {
            data = this.object
        }

        return this.perform_request(
            'PATCH', 
            id, 
            data
        )
    }

    delete(id = 0) {
        return this.perform_request(
            'DELETE',
            id
        )
    }
}

module.exports = {
    Base: Base
}
